# Copyright (c) Meta Platforms, Inc. and affiliates.
# All rights reserved.
#
# This source code is licensed under the terms described in the LICENSE file in
# the root directory of this source tree.

import argparse
import json
from pathlib import Path

import yaml

from llama_stack.cli.subcommand import Subcommand
from llama_stack.distribution.utils.config_dirs import BUILDS_BASE_DIR

from llama_stack.distribution.utils.exec import run_with_pty
from llama_stack.distribution.datatypes import *  # noqa: F403
import os


class StackConfigure(Subcommand):
    """Llama cli for configuring llama toolchain configs"""

    def __init__(self, subparsers: argparse._SubParsersAction):
        super().__init__()
        self.parser = subparsers.add_parser(
            "configure",
            prog="llama stack configure",
            description="configure a llama stack distribution",
            formatter_class=argparse.RawTextHelpFormatter,
        )
        self._add_arguments()
        self.parser.set_defaults(func=self._run_stack_configure_cmd)

    def _add_arguments(self):
        self.parser.add_argument(
            "config",
            type=str,
            help="Path to the build config file (e.g. ~/.llama/builds/<image_type>/<name>-build.yaml). For docker, this could also be the name of the docker image. ",
        )

        self.parser.add_argument(
            "--output-dir",
            type=str,
            help="Path to the output directory to store generated run.yaml config file. If not specified, will use ~/.llama/build/<image_type>/<name>-run.yaml",
        )

    def _run_stack_configure_cmd(self, args: argparse.Namespace) -> None:
        import pkg_resources

        from llama_stack.distribution.build import ImageType
        from termcolor import cprint

        docker_image = None
        conda_dir = Path(os.getenv("CONDA_PREFIX")).parent / f"llamastack-{args.config}"
        build_config_file = Path(conda_dir) / f"{args.config}-build.yaml"

        if not build_config_file.exists():
            cprint(
                f"Could not find {build_config_file}. Trying docker image name instead...",
                color="green",
            )
            docker_image = args.config

            builds_dir = BUILDS_BASE_DIR / ImageType.docker.value
            if args.output_dir:
                builds_dir = Path(output_dir)
            os.makedirs(builds_dir, exist_ok=True)

            script = pkg_resources.resource_filename(
                "llama_stack", "distribution/configure_container.sh"
            )
            script_args = [script, docker_image, str(builds_dir)]

            return_code = run_with_pty(script_args)

            # we have regenerated the build config file with script, now check if it exists
            if return_code != 0:
                self.parser.error(
                    f"Can not find {build_config_file}. Please run llama stack build first or check if docker image exists"
                )

            build_name = docker_image.removeprefix("llamastack-")
            saved_file = str(builds_dir / f"{build_name}-run.yaml")
            cprint(
                f"YAML configuration has been written to {saved_file}. You can now run `llama stack run {saved_file}`",
                color="green",
            )
            return

        with open(build_config_file, "r") as f:
            build_config = BuildConfig(**yaml.safe_load(f))

        self._configure_llama_distribution(build_config, args.output_dir)

    def _configure_llama_distribution(
        self,
        build_config: BuildConfig,
        output_dir: Optional[str] = None,
    ):
        from llama_stack.distribution.configure import configure_api_providers
        from llama_stack.distribution.utils.serialize import EnumEncoder
        from termcolor import cprint

        builds_dir = BUILDS_BASE_DIR / build_config.image_type
        if output_dir:
            builds_dir = Path(output_dir)
        os.makedirs(builds_dir, exist_ok=True)
        image_name = build_config.name.replace("::", "-")
        run_config_file = builds_dir / f"{image_name}-run.yaml"

        if run_config_file.exists():
            cprint(
                f"Configuration already exists at `{str(run_config_file)}`. Will overwrite...",
                "yellow",
                attrs=["bold"],
            )
            config = StackRunConfig(**yaml.safe_load(run_config_file.read_text()))
        else:
            config = StackRunConfig(
                built_at=datetime.now(),
                image_name=image_name,
                apis_to_serve=[],
                provider_map={},
            )

        config = configure_api_providers(config, build_config.distribution_spec)

        config.docker_image = (
            image_name if build_config.image_type == "docker" else None
        )
        config.conda_env = image_name if build_config.image_type == "conda" else None

        with open(run_config_file, "w") as f:
            to_write = json.loads(json.dumps(config.dict(), cls=EnumEncoder))
            f.write(yaml.dump(to_write, sort_keys=False))

        cprint(
            f"> YAML configuration has been written to {run_config_file}.",
            color="blue",
        )

        cprint(
            f"You can now run `llama stack run {image_name} --port PORT` or `llama stack run {run_config_file} --port PORT`",
            color="green",
        )
